 /*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        CreamProxy: an extension of AbsCreamProxy to perform SOAP communications and authentication with a CREAM service

        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: CreamProxy.h,v 1.40.2.13 2007/04/13 14:28:08 adorigo Exp $
*/

#ifndef _GLITE_CE_CREAM_CLIENT_API_SOAPPROXY_CREAMPROXY_H_
#define _GLITE_CE_CREAM_CLIENT_API_SOAPPROXY_CREAMPROXY_H_

#include "glite/ce/cream-client-api-c/cream_client_soapStub.h"
#include "glite/ce/cream-client-api-c/ExceptionFactory.h"
#include "glite/ce/cream-client-api-c/JobInfo.h"

#include <cstdlib>
#include <vector>
#include <string>
#include <map>
#include "log4cpp/Category.hh"

extern "C" {
#undef IOV_MAX
#include "glite/security/glite_gsplugin.h"
};

#define SOAP_CONNECT_TIMEOUT 60
#define SOAP_RECV_TIMEOUT 60
#define SOAP_SEND_TIMEOUT 60

namespace glite {
  namespace ce {
    namespace cream_client_api {
      namespace soap_proxy {
      
	/// A Class implementing the CREAM communication protocol.
	/**
        	CreamProxy is the CREAM C++ client that hides the user from the details
		of SOAP communication.\n
		It provides all the methods needed to communicate with a 
		CREAM service; these methods are both administrative and user-oriented.\n             
		In order to interact with a remote CREAM service, a CreamProxy object 
		must be initially instantiated with a boolean argument; then authentication
		of this object must be setup by calling Authenticate() method.\n
		The boolean argument of the CreamProxy constructor indicates if the job submissions will be done using 
		automatic delegation or not.
		If the user didn't choose the automatic delegation (argument of CreamProxy's constructor is false),
		before to perform a submission he/she has to delegate a proxy
		by using the Delegate() method.\n
		Once a CreamProxy object has been created and authenticated (and a proxy delegation has been done if
		the user choosen to do that manually) it is ready to interact with CREAM.\n
		At this point the user, tipically, could like to submit a job. This can be performed using the couple of methods 
		Register() / Start() (or only the first one if using autostart method).\n
		If the user already owns jobs somewhere, he/she can use other methods like List(), Cancel(), Suspend() etc.
		to manipulate them.
	
		The exceptions raised by some methods are due to errors occurred in the client code (like auth_ex or soap_ex) or 
		to faults raised by the CREAM service as response to a client's request.
		In both cases the exception's method what() returns a string describing, in a human readable form, the cause
		of the exception.\n\n
		Please note that all exceptions inherit from std::exception.
	*/
	class CreamProxy {

	private:

	  class safe_reset_authn {
	    CreamProxy* theProxy;
	  public:
	    safe_reset_authn(CreamProxy* p) throw(glite::ce::cream_client_api::soap_proxy::auth_ex&, glite::ce::cream_client_api::soap_proxy::soap_ex&);
	    ~safe_reset_authn() throw();
	  };

	  struct soap* m_soap;

	  int m_ctimeout;
	  int m_iotimeout;

          log4cpp::Category *m_log_dev;
          std::string m_delegation_id;
          bool   m_automatic_delegation;

	  glite_gsplugin_Context m_ctx;
	  bool m_setsoapheader;
	  std::string m_iceid;

	  std::string m_vo;
	  std::vector<std::string> m_cert_info_vec;
	  std::vector<std::string> m_tmp_cert_info_vec;
	  std::string m_method_name, m_error_code, m_description, m_fault_code;
	  std::string m_distinguished_name;
	  std::string m_certfile;

	  int m_nbio;

	  void clearSoap( void );
	  void makeSoap( void ) throw(glite::ce::cream_client_api::soap_proxy::soap_ex&, glite::ce::cream_client_api::soap_proxy::auth_ex&);

	public:

	  /// Creates a CreamProxy object
	  /**
	  	This constructor requires a mandatory boolean argument that specifies if
		the calls of Register() method (the first step for job submission, see below)
		will also automatically delegate a proxy or not. \n
		(The constructor internally instantiates a
		SOAP data structure by calling gSOAP's specific memory allocation functions, but this
		is completely transparent for the user.)
		\see Delegate()
		\see Register()
		\param automatic_delegation Boolean that specifies if a proxy has to be automatically delegated
		\throw glite::ce::cream_client_api::soap_proxy::soap_ex& If a serious error occurs when creating the
		internal SOAP data structure; this is severe (the operating system probably failed a malloc() call) 
		and it is better to give up and exit.
	  */
	  CreamProxy(const bool automatic_delegation, const int timeout = 30) 
	    throw(glite::ce::cream_client_api::soap_proxy::soap_ex&);

	  /// CreamProxy's destructor
	  /**

	  */
	  virtual ~CreamProxy(void) throw();
	  
	  /// Sets up the authentication
	  /**
	  	This method sets up the client certificate proxy file that will be
		used for any authenticated communication with the CREAM service
		\see certUtil::getDN()
		\param proxyfile The client certificate proxy file
		\return The Virtual organisation name contained in the user certificate
		\throw glite::ce::cream_client_api::soap_proxy::auth_ex& If the proxy contained
		in the file is not valid (e.g. expired) or corrupted, or in general unusable for
		authentication.
	  */
	  virtual std::string Authenticate(const std::string& proxyfile)
	    throw(glite::ce::cream_client_api::soap_proxy::auth_ex&);

	  /// Deprecated: this method has no effect
          virtual void printDebug(bool dbg) { /*DBG = dbg;*/ }
	  
	  /// Suspends one or more jobs
	  /**
	  	This method sends a JobSuspend request to CREAM service. This can be done only for jobs whose
		status is PENDING, IDLE, RUNNING or REALLY_RUNNING.
		\param service The endpoint where the CREAM service is running (the format must be
		https://[host]:[port]/ce-cream/services/CREAM)
		\param jobids A vector of strings representing the job IDs (as returned by the Register call) to suspend.
		If the vector is empty the action is taken for all jobs belonging to the current user (whose identity is 
		extracted from the proxy certificate).
		\throw glite::ce::cream_client_api::cream_exceptions::JobUnknownException& If trying to suspend an
		unexistent job
		\throw glite::ce::cream_client_api::cream_exceptions::AuthorizationException& If the user is not authorized
		to perform the operation
		\throw glite::ce::cream_client_api::cream_exceptions::AuthenticationException& If the authentication failed
		 (e.g. the user certificate is not recognized/accepted by the service)
		\throw glite::ce::cream_client_api::cream_exceptions::JobStatusInvalidException& If the job is in a status
		incompatible with suspend (see above)
		\throw glite::ce::cream_client_api::cream_exceptions::OperationNotSupportedException& IF the underlying
		batch system doesn't support the suspend operation
		\throw glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException& If at least one of the 
		arguments of the method are invalid for the CREAM service (actually cannot happen in this case,
		since even an empty vector is accepted)
		\throw glite::ce::cream_client_api::cream_exceptions::GenericException& For any other fault 
		raised by the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::InternalException& If something went wrong in the
		client code
		\throw glite::ce::cream_client_api::cream_exceptions::BaseException& If the fault raised by the CREAM
		service is not recognized by the client
	  */
	  virtual void Suspend(const char* service, const std::vector<std::string>& jobids) 
	    throw(glite::ce::cream_client_api::cream_exceptions::BaseException&,
		  glite::ce::cream_client_api::cream_exceptions::JobUnknownException&,
		  glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException&,
		  glite::ce::cream_client_api::cream_exceptions::OperationNotSupportedException&,
		  glite::ce::cream_client_api::cream_exceptions::JobStatusInvalidException&,
		  glite::ce::cream_client_api::cream_exceptions::GenericException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthenticationException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthorizationException&,
		  glite::ce::cream_client_api::cream_exceptions::InternalException&,
		  glite::ce::cream_client_api::cream_exceptions::ConnectionTimeoutException&,
		  glite::ce::cream_client_api::soap_proxy::auth_ex&);
	  
	  /// Resumes one or more suspended jobs
	  /**
	  	This method sends a JobResume request to CREAM service. The job(s) to resume must be in HELD (suspended) state.
		\param service The endpoint where the CREAM service is running (the format must be
		https://[host]:[port]/ce-cream/services/CREAM)
		\param jobids A vector of strings representing the job IDs (as returned by the Register call) to resume.
		If the vector is empty the action is taken for all jobs belonging to the current user (whose identity is
		extracted from the proxy certificate).
		\throw glite::ce::cream_client_api::cream_exceptions::JobUnknownException& If trying to resume an
		unexistent job
		\throw glite::ce::cream_client_api::cream_exceptions::AuthorizationException& If the user is not authorized
		to perform the operation
		\throw glite::ce::cream_client_api::cream_exceptions::AuthenticationException& If the authentication failed
		 (e.g. the user certificate is not recognized/accepted by the service)
		\throw glite::ce::cream_client_api::cream_exceptions::JobStatusInvalidException& If the job is in a status
		incompatible with resume (e.g. the job is not suspended)
		\throw glite::ce::cream_client_api::cream_exceptions::OperationNotSupportedException& If the underlying
		batch system doesn't support the resume operation
		\throw glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException& If at least one of the
		arguments of the method are invalid for the CREAM service (actually this cannot happen in this case,
		since even an empty vector is accepted)
		\throw glite::ce::cream_client_api::cream_exceptions::GenericException& For any other fault
		raised by the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::InternalException& If something went wrong in the
		client code
		\throw glite::ce::cream_client_api::cream_exceptions::BaseException& If the fault raised by the CREAM
		service is not recognized by the client
	  */
	  virtual void Resume(const char* service, const std::vector<std::string>& jobids)
	    throw(glite::ce::cream_client_api::cream_exceptions::BaseException&,
		  glite::ce::cream_client_api::cream_exceptions::JobUnknownException&,
		  glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException&,
		  glite::ce::cream_client_api::cream_exceptions::OperationNotSupportedException&,
		  glite::ce::cream_client_api::cream_exceptions::JobStatusInvalidException&,
		  glite::ce::cream_client_api::cream_exceptions::GenericException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthenticationException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthorizationException&,
		  glite::ce::cream_client_api::cream_exceptions::InternalException&,
		  glite::ce::cream_client_api::cream_exceptions::ConnectionTimeoutException&,
		  glite::ce::cream_client_api::soap_proxy::auth_ex&);
		  
	  /// Cancels one or more jobs
	  /**
	  	This method sends a JobCancel request to CREAM service. The job(s) to cancel must be in REGISTERED, PENDING, IDLE,
		RUNNING, REALLY_RUNNING or HELD state (see job_statuses namespace for all possible job states).
		\see job_statuses
		\param service The endpoint where the CREAM service is running (the format must be
		https://[host]:[port]/ce-cream/services/CREAM)
		\param jobids A vector of strings representing the job IDs (as returned by the Register() method) to cancel.
		If the vector is empty the action is applied to all jobs belonging to the current user (whose identity is
		extracted from the proxy certificate set with Authenticate() method).
		\throw glite::ce::cream_client_api::cream_exceptions::JobUnknownException& If trying to cancel an
		unexistent job
		\throw glite::ce::cream_client_api::cream_exceptions::AuthorizationException& If the user is not authorized
		to perform the operation
		\throw glite::ce::cream_client_api::cream_exceptions::AuthenticationException& If the authentication went
		wrong (e.g. the user certificate is not recognized/accepted by the service)
		\throw glite::ce::cream_client_api::cream_exceptions::JobStatusInvalidException& If the job is in a status
		incompatible with cancel (i.e. the job is DONE_FAILED or DONE_OK)
		\throw glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException& If at least one of the 
		arguments of the method are invalid for the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::GenericException& For any other fault
		raised by the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::InternalException& If something went wrong in the
		client code
		\throw glite::ce::cream_client_api::cream_exceptions::BaseException& If the fault raised by the CREAM
		service is not recognized by the client
	  */
	  virtual void Cancel(const char* service, const std::vector<std::string>& jobids)
	    throw(glite::ce::cream_client_api::cream_exceptions::BaseException&,
		  glite::ce::cream_client_api::cream_exceptions::JobUnknownException&,
		  glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException&,
		  glite::ce::cream_client_api::cream_exceptions::JobStatusInvalidException&,
		  glite::ce::cream_client_api::cream_exceptions::GenericException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthenticationException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthorizationException&,
		  glite::ce::cream_client_api::cream_exceptions::InternalException&,
		  glite::ce::cream_client_api::cream_exceptions::ConnectionTimeoutException&,
		  glite::ce::cream_client_api::soap_proxy::auth_ex&);
	  
/* 	  virtual std::string Assess(const char* service,
				     const std::string& JDL, const std::string& param) 
	    throw(glite::ce::cream_client_api::cream_exceptions::BaseException&,
		  glite::ce::cream_client_api::cream_exceptions::JobUnknownException&,
		  glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException&,
		  glite::ce::cream_client_api::cream_exceptions::OperationNotSupportedException&,
		  glite::ce::cream_client_api::cream_exceptions::NoSuitableResourceException&,
		  glite::ce::cream_client_api::cream_exceptions::GridProxyDelegationException&,
		  glite::ce::cream_client_api::cream_exceptions::JobStatusInvalidException&,
		  glite::ce::cream_client_api::cream_exceptions::GenericException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthenticationException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthorizationException&,
		  glite::ce::cream_client_api::cream_exceptions::InternalException&); */
	  
/* 	  virtual void Signal(const char* service,
			      const std::vector<std::string>& jobid, 
			      const std::string& sig) 
	    throw(glite::ce::cream_client_api::cream_exceptions::BaseException&,
		  glite::ce::cream_client_api::cream_exceptions::JobUnknownException&,
		  glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException&,
		  glite::ce::cream_client_api::cream_exceptions::OperationNotSupportedException&,
		  glite::ce::cream_client_api::cream_exceptions::NoSuitableResourceException&,
		  glite::ce::cream_client_api::cream_exceptions::GridProxyDelegationException&,
		  glite::ce::cream_client_api::cream_exceptions::JobStatusInvalidException&,
		  glite::ce::cream_client_api::cream_exceptions::GenericException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthenticationException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthorizationException&,
		  glite::ce::cream_client_api::cream_exceptions::InternalException&);
	   */
	  
	  /// Retrieves the list of jobs
	  /**
	  	This method sends a JobList request to the CREAM service. CREAM sends back a list of all (not purged) jobs
		 submitted by the considered user. All strings representing the returned jobs are put in the
		second argument of the method.
		\param service The endpoint where the CREAM service is running (the format must be
		https://[host]:[port]/ce-cream/services/CREAM)
		\param returned_jobids The variable that will contain all the jobids returned by CREAM
		\throw glite::ce::cream_client_api::cream_exceptions::AuthorizationException& If the user is not authorized
		to perform the operation
		\throw glite::ce::cream_client_api::cream_exceptions::AuthenticationException& If the authentication failed
		 (e.g. the user certificate is not recognized/accepted by the service)
		\throw glite::ce::cream_client_api::cream_exceptions::GenericException& For any other fault
		raised by the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::InternalException& If something went wrong in the
		client code
		\throw glite::ce::cream_client_api::cream_exceptions::BaseException& If the fault raised by the CREAM
		service is not recognized by the client
	  */
	  virtual void List(const char* service, std::vector<std::string>& returned_jobids)
	    throw(glite::ce::cream_client_api::cream_exceptions::BaseException&,
		  glite::ce::cream_client_api::cream_exceptions::GenericException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthenticationException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthorizationException&,
		  glite::ce::cream_client_api::cream_exceptions::InternalException&,
		  glite::ce::cream_client_api::cream_exceptions::ConnectionTimeoutException&,
		  glite::ce::cream_client_api::soap_proxy::auth_ex&);

	  /// Purges one or more jobs
	  /**
	  	This method sends a JobPurge request to CREAM service. After a job has been purged,
		it can't be managed anymore.\n
		The job(s) to purge must be in a terminal state
		(DONE_OK, DONE_FAILED, CANCELLED, ABORTED).
		\param service The endpoint where the CREAM service is running (the format must be
		https://[host]:[port]/ce-cream/services/CREAM)
		\param jobids A vector of strings representing the job IDs (as returned by the Register call) to purge.
		If the vector is empty the action is taken for all jobs belonging to the current user (whose identity is 
		extracted from the proxy certificate).
		\throw glite::ce::cream_client_api::cream_exceptions::JobUnknownException& If trying to purge an
		unexistent job
		\throw glite::ce::cream_client_api::cream_exceptions::AuthorizationException& If the user is not authorized
		to perform the operation
		\throw glite::ce::cream_client_api::cream_exceptions::AuthenticationException& If the authentication failed
		 (e.g. the user certificate is not recognized/accepted by the service)
		\throw glite::ce::cream_client_api::cream_exceptions::JobStatusInvalidException& If the job is in a status
		incompatible with purge (e.g. the job is RUNNING)
		\throw glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException& If at least one of the
		arguments of the method is invalid for the CREAM service (actually this cannot happen in this case,
		because even an empty vector is accepted)
		\throw glite::ce::cream_client_api::cream_exceptions::GenericException& For any other fault
		raised by the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::InternalException& If something went wrong in the
		client code
		\throw glite::ce::cream_client_api::cream_exceptions::BaseException& If the fault raised by the CREAM
		service is not recognized by the client
	  */
	  virtual void Purge(const char* service, const std::vector<std::string>& jobids)
	    throw(glite::ce::cream_client_api::cream_exceptions::BaseException&,
		  glite::ce::cream_client_api::cream_exceptions::JobUnknownException&,
		  glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException&,
		  glite::ce::cream_client_api::cream_exceptions::GenericException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthenticationException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthorizationException&,
		  glite::ce::cream_client_api::cream_exceptions::InternalException&,
		  glite::ce::cream_client_api::cream_exceptions::ConnectionTimeoutException&,
		  glite::ce::cream_client_api::soap_proxy::auth_ex&);
	  
	  /// Registers a job in the CREAM service
	  /**
	  	This method sends a JobRegister request to the CREAM service. It is the first of the two
		steps needed for job submission.
		The register sets up everything in the remote environment to properly submit the job: delegation handshake, preparation
		of the input sandbox upload directory.\n
		The Register should be followed by a Start call in order to actually submit the job to the batch system.
		The user
		can also choose to tell CREAM the the JobRegister should automatically start (submit) the job if there
		are no input sandbox files to upload.
		\param service The endpoint where the CREAM service is running (the format must be
		https://[host]:[port]/ce-cream/services/CREAM)
		\param delegation_service The endpoint where the CREAM service is running (the format usually is
		https://[host]:[port]/ce-cream/services/CREAMDelegation)
		\param delegationID A string representing an identifier for a previously delegated proxy certificate.
		This string can be empty (""); in this case the Register will automatically perform a Delegation choosing
		an unique ID and using this for the delegationID variable
		\param jdl The JDL describing the job to submit
		\param certfile The user proxy certificate file
		\param register_output A vector (of only two string elements) containing the upload directory for the
		input sandbox of the job and the job identifier
		\param lease_time a parameter conceived only for usage by ICE. If you're developing a Command Line Interface please set it to 0
		\param autostart This parameter tells CREAM if it should automatically call the start method
		after the Register (this is the tipical scenario in the case the UI has no sandbox to upload)
		\throw glite::ce::cream_client_api::cream_exceptions::AuthorizationException& If the user is not authorized
		to perform the operation
		\throw glite::ce::cream_client_api::cream_exceptions::AuthenticationException& If the authentication failed
		 (e.g. the user certificate is not recognized/accepted by the service)
		\throw glite::ce::cream_client_api::cream_exceptions::GridProxyDelegationException& If the CREAM raised a fault concerning
		a failed delegation of the user proxy certificate
		\throw glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException& If at least one of the
		arguments of the method are invalid for the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::JobSubmissionDisabledException& If the CREAM service
		has job
		submissions disabled (see CreamProxy::DisableAcceptJobSubmissions and CreamProxy::EnableAcceptJobSubmissions)
		\throw glite::ce::cream_client_api::cream_exceptions::GenericException& For any other fault
		raised by the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::InternalException& If something went wrong in the
		client code
		\throw glite::ce::cream_client_api::cream_exceptions::BaseException& If the fault raised by the CREAM
		service is not recognized by the client
		\throw glite::ce::cream_client_api::cream_exceptions::DelegationException& If the Delegate method (automatically
		called if the autostart param is true) raised this exception
	  */
	  virtual void Register(const char* service,
				const char* delegation_service,
				std::string& delegationID,
				const std::string& jdl,
				const std::string& certfile,
				std::vector<std::string>& register_output,
				int& lease_time,
				const bool autostart = false)
	    throw(glite::ce::cream_client_api::cream_exceptions::BaseException&,
		  glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException&,
		  glite::ce::cream_client_api::cream_exceptions::GridProxyDelegationException&,
		  glite::ce::cream_client_api::cream_exceptions::JobSubmissionDisabledException&,
		  glite::ce::cream_client_api::cream_exceptions::GenericException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthenticationException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthorizationException&,
		  glite::ce::cream_client_api::cream_exceptions::DelegationException&,
		  glite::ce::cream_client_api::cream_exceptions::InternalException&,
		  glite::ce::cream_client_api::cream_exceptions::ConnectionTimeoutException&,
		  glite::ce::cream_client_api::soap_proxy::auth_ex&);

	  /// Triggers the real job submission
	  /**
	  	After a Register has been called (and the input sandbox has been uploaded) the Start should be
		invoked in order to actually submit the job to the batch system.
		\param service The endpoint where the CREAM service is running (the format must be
		https://[host]:[port]/ce-cream/services/CREAM)
		\param jobID The identifier (as returned by the Register) of the job to submit
		\throw glite::ce::cream_client_api::cream_exceptions::AuthorizationException& If the user is not authorized
		to perform the operation
		\throw glite::ce::cream_client_api::cream_exceptions::AuthenticationException& If the authentication (went
		wrong (e.g. the user certificate is not recognized/accepted by the service)
		\throw glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException& If at least one of the
		arguments of the method are invalid for the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::GenericException& For any other fault
		raised by the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::JobSubmissionDisabledException& The CREAM service has the
		submission disabled (see CreamProxy::DisableAcceptJobSubmissions and CreamProxy::EnableAcceptJobSubmissions)
		\throw glite::ce::cream_client_api::cream_exceptions::JobStatusInvalidException& If the job is in a state incompatible with submission
		\throw glite::ce::cream_client_api::cream_exceptions::JobUnknownException& If trying to start an unexistent job
		\throw glite::ce::cream_client_api::cream_exceptions::InternalException& If something went wrong in the
		client code
		\throw glite::ce::cream_client_api::cream_exceptions::BaseException& If the fault raised by the CREAM
		service is not recognized by the client
	  */
	  virtual void Start(const char* service, const std::string& jobID)
	    throw(glite::ce::cream_client_api::cream_exceptions::BaseException&,
		  glite::ce::cream_client_api::cream_exceptions::JobUnknownException&,
		  glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException&,
		  glite::ce::cream_client_api::cream_exceptions::JobStatusInvalidException&,
		  glite::ce::cream_client_api::cream_exceptions::GenericException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthenticationException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthorizationException&,
		  glite::ce::cream_client_api::cream_exceptions::InternalException&,
		  glite::ce::cream_client_api::cream_exceptions::ConnectionTimeoutException&,
		  glite::ce::cream_client_api::cream_exceptions::JobSubmissionDisabledException&,
		  glite::ce::cream_client_api::soap_proxy::auth_ex&);

	  /// Retrieves information about registered jobs
	  /**
	  	This method query the CREAM service for all available information about one or more jobs.
		\param service The endpoint where the CREAM service is running (the format must be
		https://[host]:[port]/ce-cream/services/CREAM)
		\param JID A vector of all the job identifiers the user needs information about
		\param STATES A vector of all possible states of jobs. The output is returned only for
		jobs that are in one of the states specified in STATES
		\param target A vector of JobInfo objects (one for each job); each object contains job information (see JobInfo class description)
		\param since A date and time (expressed as number of seconds since the epoch) that filters out all the jobs
		that have been registered before this date
		\param to A date and time (expressed as number of seconds since the epoch) that filters out all the jobs
		that have been registered after this date
		this SOAP structure
		\throw glite::ce::cream_client_api::cream_exceptions::AuthorizationException& If the user is not authorized
		to perform the operation
		\throw glite::ce::cream_client_api::cream_exceptions::AuthenticationException& If the authentication failed
		 (e.g. the user certificate is not recognized/accepted by the service)
		\throw glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException& If at least one of the
		arguments of the method are invalid for the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::GenericException& For any other fault
		raised by the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::JobUnknownException& If trying to get information
		for one or more unexistent jobs
		\throw glite::ce::cream_client_api::cream_exceptions::InternalException& If something went wrong in the
		client code
		\throw glite::ce::cream_client_api::cream_exceptions::BaseException& If the fault raised by the CREAM
		service is not recognized by the client
		\throw glite::ce::cream_client_api::soap_proxy::invalidTimestamp_ex& If the date and time specified for 'to' is earlier of that one specified for 'since'
	  */
	  virtual void Info(const char* service,
			    const std::vector<std::string>& JID,
			    const std::vector<std::string>& STATES,
		   	    std::vector<glite::ce::cream_client_api::soap_proxy::JobInfo>& target,
			    const int since=-1,
			    const int to=-1
		 	    )
	    throw(glite::ce::cream_client_api::cream_exceptions::BaseException&,
		  glite::ce::cream_client_api::cream_exceptions::JobUnknownException&,
		  glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException&,
		  glite::ce::cream_client_api::cream_exceptions::GenericException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthenticationException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthorizationException&,
		  glite::ce::cream_client_api::cream_exceptions::InternalException&,
		  glite::ce::cream_client_api::cream_exceptions::ConnectionTimeoutException&,
		  glite::ce::cream_client_api::soap_proxy::invalidTimestamp_ex&,
		  glite::ce::cream_client_api::soap_proxy::auth_ex&);

	  /// Retrieves information about registered jobs
	  /**
	  	This method is like Info but faster because transfer less information.
		\param service The endpoint where the CREAM service is running (the format must be
		https://[host]:[port]/ce-cream/services/CREAM)
		\param JID A vector of all job identifiers the user needs information about
		\param STATES A vector of all possible states of jobs. The information is returned only for those
		jobs that are in one of the states specified in STATES
		\param target A vector of Status objects (one for each job); each object contains essential job information (see Status class description). This vector is not cleared before to be filled up, and Status objects are appended to it.
		\param since A date and time (expressed as number of seconds since the epoch) that filters out all the jobs
		that have been register before this date
		\param to A date and time (expressed as number of seconds since the epoch) that filters out all the jobs
		that have been register after this date
		\throw glite::ce::cream_client_api::cream_exceptions::AuthorizationException& If the user is not authorized
		to perform the operation
		\throw glite::ce::cream_client_api::cream_exceptions::AuthenticationException& If the authentication failed
		 (e.g. the user certificate is not recognized/accepted by the service)
		\throw glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException& If at least one of the
		arguments of the method are invalid for the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::GenericException& For any other fault
		raised by the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::JobUnknownException& If trying to get information
		for one or more unexistent jobs
		\throw glite::ce::cream_client_api::cream_exceptions::InternalException& If something went wrong in the
		client code
		\throw glite::ce::cream_client_api::cream_exceptions::BaseException& If the fault raised by the CREAM
		service is not recognized by the client
		\throw glite::ce::cream_client_api::soap_proxy::invalidTimestamp_ex& If the date and time specified for 'to' is earlier
		than that one specified for 'since'
	  */
	  virtual void Status(const char* service,
			      const std::vector<std::string>& JID,
			      const std::vector<std::string>& STATES,
			      std::vector<glite::ce::cream_client_api::soap_proxy::Status>& target,
			      const int since=-1,
			      const int to=-1)
	    throw(glite::ce::cream_client_api::cream_exceptions::BaseException&,
		  glite::ce::cream_client_api::cream_exceptions::JobUnknownException&,
		  glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException&,
		  glite::ce::cream_client_api::cream_exceptions::GenericException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthenticationException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthorizationException&,
		  glite::ce::cream_client_api::cream_exceptions::InternalException&,
		  glite::ce::cream_client_api::cream_exceptions::ConnectionTimeoutException&,
		  glite::ce::cream_client_api::soap_proxy::invalidTimestamp_ex&,
		  glite::ce::cream_client_api::soap_proxy::auth_ex&);

/*	  virtual std::string SimpleStatus(const char* service,
				           const std::string& JID)
	    throw(glite::ce::cream_client_api::cream_exceptions::BaseException&,
		  glite::ce::cream_client_api::cream_exceptions::JobUnknownException&,
		  glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException&,
		  glite::ce::cream_client_api::cream_exceptions::OperationNotSupportedException&,
		  glite::ce::cream_client_api::cream_exceptions::NoSuitableResourceException&,
		  glite::ce::cream_client_api::cream_exceptions::GridProxyDelegationException&,
		  glite::ce::cream_client_api::cream_exceptions::JobStatusInvalidException&,
		  glite::ce::cream_client_api::cream_exceptions::GenericException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthenticationException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthorizationException&,
		  glite::ce::cream_client_api::cream_exceptions::InternalException&,
		  glite::ce::cream_client_api::soap_proxy::soap_ex&);*/

	  /// Retrieves the URL of the CEMon service coupled to the considered CREAM service
	  /**
	  	This method allows the user to know which is the CEMon connected to a specified CREAM service.
		\param cream_service The URL of the CREAM service the user want to know the CEMon URL of
		\param cemon_service The variable receiving the URL of the CEMon connected to the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::AuthorizationException& If the user is not authorized
		to perform the operation
		\throw glite::ce::cream_client_api::cream_exceptions::AuthenticationException& If the authentication failed
		 (e.g. the user certificate is not recognized/accepted by the service)
		\throw glite::ce::cream_client_api::cream_exceptions::GenericException& For any other fault
		raised by the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::BaseException& If the fault raised by the CREAM
		service is not recognized by the client
		\throw glite::ce::cream_client_api::cream_exceptions::InternalException& If something went wrong in the
		client code
	  */
	  virtual void GetCEMonURL(const char* cream_service, std::string& cemon_service)
	     throw(glite::ce::cream_client_api::cream_exceptions::BaseException&,
		   glite::ce::cream_client_api::cream_exceptions::GenericException&,
		   glite::ce::cream_client_api::cream_exceptions::AuthenticationException&,
		   glite::ce::cream_client_api::cream_exceptions::AuthorizationException&,
		   glite::ce::cream_client_api::cream_exceptions::InternalException&,
		   glite::ce::cream_client_api::cream_exceptions::ConnectionTimeoutException&,
		   glite::ce::cream_client_api::soap_proxy::auth_ex&);

	  /// Delegates a proxy to the CREAM service
	  /**
	  	This method delegates the client proxy certificate to the CREAM service
		\param id The delegation identifier (an arbitrary string chosen by the user)
		\param service The endpoint where the CREAM service is running (the format must be
		https://[host]:[port]/ce-cream/services/CREAMDelegation)
		\param certfile The file containing the client proxy certificate to delegate
		\throw glite::ce::cream_client_api::cream_exceptions::DelegationException& If the delegation method failed at server side
		\throw glite::ce::cream_client_api::cream_exceptions::InternalException& If something went wrong in the
		client code
	  */
	  virtual void Delegate(const std::string& id,
				const std::string& service,
				const std::string& certfile)
	    throw(glite::ce::cream_client_api::cream_exceptions::DelegationException&,
		  glite::ce::cream_client_api::cream_exceptions::InternalException&,
		  glite::ce::cream_client_api::cream_exceptions::ConnectionTimeoutException&,
		  glite::ce::cream_client_api::soap_proxy::auth_ex&);

	  virtual void Delegate_New(const std::string& id,
                                const std::string& service,
                                const std::string& certfile,
				std::string& delegationProxy
				)
            throw(glite::ce::cream_client_api::cream_exceptions::DelegationException&,
                  glite::ce::cream_client_api::cream_exceptions::InternalException&,
                  glite::ce::cream_client_api::cream_exceptions::ConnectionTimeoutException&,
                  glite::ce::cream_client_api::soap_proxy::auth_ex&);	
		  
	private:
          virtual void DelegateRenew(const std::string& id,
				     const std::string& service,
				     const std::string& certfile)
	    throw(glite::ce::cream_client_api::cream_exceptions::DelegationException&,
		  glite::ce::cream_client_api::cream_exceptions::InternalException&,
		  glite::ce::cream_client_api::cream_exceptions::ConnectionTimeoutException&,
		  glite::ce::cream_client_api::soap_proxy::auth_ex&);

	public:
	  /// Pings a CREAM service
	  /**
	  	This method lets the client know if a CREAM service is reachable or not
		\param service The endpoint where the CREAM service is running (the format must be
		https://[host]:[port]/ce-cream/services/CREAM)
		\return True if the remote CREAM service is reachable, false otherwise
	  */
	  virtual bool Ping( const char* service ) throw(glite::ce::cream_client_api::soap_proxy::auth_ex&, glite::ce::cream_client_api::cream_exceptions::ConnectionTimeoutException&);

	  /// Sets the Lease time increment
	  /**
	  	This method sets the lease time increment for one or more jobs. The CREAM service can accept the lease renewal or not.
		Then in the third argument the user will find the actual lease increment for each jobid.
		\param service The endpoint where the CREAM service is running (the format must be
		https://[host]:[port]/ce-cream/services/CREAM)
		\param jobIDs The array of job identifiers the user want increase the lease time of
		\param leaseTimeIncrement The lease time increment to apply to all jobIDs
		\param leaseTimes A map job identifiers -> new lease times (as set by CREAM)
		\throw glite::ce::cream_client_api::cream_exceptions::AuthenticationException& If the authentication (went
		wrong (e.g. the user certificate is not recognized/accepted by the service)
		\throw glite::ce::cream_client_api::cream_exceptions::AuthorizationException& If the user is not authorized
		to perform the operation
		\throw glite::ce::cream_client_api::cream_exceptions::GenericException& For any other fault
		raised by the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::BaseException& If the fault raised by the CREAM
		service is not recognized by the client
		\throw glite::ce::cream_client_api::cream_exceptions::InternalException& If something went wrong in the
		client code
		\throw glite::ce::cream_client_api::cream_exceptions::JobUnknownException& If the user tried to increment
		the lease time for one or more unexistent jobs
		\throw glite::ce::cream_client_api::cream_exceptions::JobStatusInvalidException& If the user tried to increment
		the lase time for one or more jobs whose status is incompatible with the operation (i.e. it doesn't make
		any sense to increment the lease time for a job in DONE_OK status)
	  */
	  virtual void Lease( const char* service,
	                      const std::vector<std::string>& jobIDs,
			      const int leaseTimeIncrement, std::map<std::string, time_t>& leaseTimes)
	    throw(glite::ce::cream_client_api::cream_exceptions::BaseException&,
		  glite::ce::cream_client_api::cream_exceptions::GenericException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthenticationException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthorizationException&,
		  glite::ce::cream_client_api::cream_exceptions::InternalException&,
		  glite::ce::cream_client_api::cream_exceptions::ConnectionTimeoutException&,
		  glite::ce::cream_client_api::cream_exceptions::JobUnknownException&,
		  glite::ce::cream_client_api::cream_exceptions::JobStatusInvalidException&,
		  glite::ce::cream_client_api::soap_proxy::auth_ex&);

	  /// Enables job submission of a CREAM service
	  /**
	   	This method turns ON the submission of the CREAM service specified in its argument
	  	\param service The endpoint where the CREAM service is running (the format must be
		https://[host]:[port]/ce-cream/services/CREAM)
		\throw glite::ce::cream_client_api::cream_exceptions::AuthenticationException& If the authentication failed
		 (e.g. the user certificate is not recognized/accepted by the service)
		\throw glite::ce::cream_client_api::cream_exceptions::AuthorizationException& If the user is not authorized
		to perform the operation
		\throw glite::ce::cream_client_api::cream_exceptions::GenericException& For any other fault
		raised by the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::BaseException& If the fault raised by the CREAM
		service is not recognized by the client
		\throw glite::ce::cream_client_api::cream_exceptions::InternalException& If something went wrong in the
		client code
	  */
	  virtual void EnableAcceptJobSubmissions( const char* service )
	    throw(glite::ce::cream_client_api::cream_exceptions::BaseException&,
		  glite::ce::cream_client_api::cream_exceptions::GenericException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthenticationException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthorizationException&,
		  glite::ce::cream_client_api::cream_exceptions::InternalException&,
		  glite::ce::cream_client_api::cream_exceptions::ConnectionTimeoutException&,
		  glite::ce::cream_client_api::soap_proxy::auth_ex&);

	  /// Disables job submissions of a CREAM service
	  /**
	   	This method turns OFF job submissions of the CREAM service specified in its argument
	  	\param service The endpoint where the CREAM service is running (the format must be
		https://[host]:[port]/ce-cream/services/CREAM)
		\throw glite::ce::cream_client_api::cream_exceptions::AuthenticationException& If the authentication failed
		 (e.g. the user certificate is not recognized/accepted by the service)
		\throw glite::ce::cream_client_api::cream_exceptions::AuthorizationException& If the user is not authorized
		to perform the operation
		\throw glite::ce::cream_client_api::cream_exceptions::GenericException& For any other fault
		raised by the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::BaseException& If the fault raised by the CREAM
		service is not recognized by the client
		\throw glite::ce::cream_client_api::cream_exceptions::InternalException& If something went wrong in the
		client code
	  */
	  virtual void DisableAcceptJobSubmissions( const char* service )
	    throw(glite::ce::cream_client_api::cream_exceptions::BaseException&,
		  glite::ce::cream_client_api::cream_exceptions::GenericException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthenticationException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthorizationException&,
		  glite::ce::cream_client_api::cream_exceptions::InternalException&,
		  glite::ce::cream_client_api::cream_exceptions::ConnectionTimeoutException&,
		  glite::ce::cream_client_api::soap_proxy::auth_ex&);

	  /// Asks if a CREAM service has job submission enabled
	  /**
	   	This method checks if the CREAM service specified in its argument is willing to receive new
		jobs or if job submissions are disabled
	  	\param service The endpoint where the CREAM service is running (the format must be
		https://[host]:[port]/ce-cream/services/CREAM)
		\return True if the remote CREAM service accepts submissions, false if it doesn't
		\throw glite::ce::cream_client_api::cream_exceptions::AuthenticationException& If the authentication failed
		 (e.g. the user certificate is not recognized/accepted by the service)
		\throw glite::ce::cream_client_api::cream_exceptions::AuthorizationException& If the user is not authorized
		to perform the operation
		\throw glite::ce::cream_client_api::cream_exceptions::GenericException& For any other fault
		raised by the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::BaseException& If the fault raised by the CREAM
		service is not recognized by the client
		\throw glite::ce::cream_client_api::cream_exceptions::InternalException& If something went wrong in the
		client code
	  */
	  virtual bool DoesAcceptJobSubmissions( const char* service )
	    throw(glite::ce::cream_client_api::cream_exceptions::BaseException&,
		  glite::ce::cream_client_api::cream_exceptions::GenericException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthenticationException&,
		  glite::ce::cream_client_api::cream_exceptions::AuthorizationException&,
		  glite::ce::cream_client_api::cream_exceptions::InternalException&,
		  glite::ce::cream_client_api::cream_exceptions::ConnectionTimeoutException&,
		  glite::ce::cream_client_api::soap_proxy::auth_ex&);

	  /// Renews the proxy of one or more jobs
	  /**
	  	This method renew a previously delegated proxy identified by an ID string
		\param delegID A string representing the identifier used when a proxy was delegated
		\param service The endpoint where the CREAM service is running (the format must be
		https://[host]:[port]/ce-cream/services/CREAM)
		\param delegService The endpoint where the CREAMDelegation service is running (the format must be
		https://[host]:[port]/ce-cream/services/CREAMDelegation)
		\param certfile The file containing the client proxy certificate to delegate
		\param JobIDs A vector containing all job identifiers to renew the proxy for
		\throw glite::ce::cream_client_api::cream_exceptions::AuthenticationException& If the authentication failed
		 (e.g. the user certificate is not recognized/accepted by the service)
		\throw glite::ce::cream_client_api::cream_exceptions::AuthorizationException& If the user is not authorized
		to perform the operation
		\throw glite::ce::cream_client_api::cream_exceptions::GenericException& For any other fault
		raised by the CREAM service
		\throw glite::ce::cream_client_api::cream_exceptions::BaseException& If the fault raised by the CREAM
		service is not recognized by the client
		\throw glite::ce::cream_client_api::cream_exceptions::InternalException& If something went wrong in the
		client code
		\throw glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException& If at least one of the
		arguments of the method are invalid for the CREAM service (e.g. if delegID="", or the vector of
		job identifiers is empty)
		\throw glite::ce::cream_client_api::cream_exceptions::JobUnknownException& If the job ID vector contains at least
		one unexistent job
		\throw glite::ce::cream_client_api::cream_exceptions::GridProxyDelegationException& If the preliminary step
		of the proxy renewal (that is a delegation process) failed
	  */
	  virtual void renewProxy(const std::string& delegID,
	  			  const std::string& service,
				  const std::string& delegService,
				  const std::string& certfile,
	  			  const std::vector<std::string>& JobIDs)
	    throw(glite::ce::cream_client_api::cream_exceptions::BaseException&,
	          glite::ce::cream_client_api::cream_exceptions::JobUnknownException&,
	          glite::ce::cream_client_api::cream_exceptions::GenericException&,
	          glite::ce::cream_client_api::cream_exceptions::AuthenticationException&,
	          glite::ce::cream_client_api::cream_exceptions::AuthorizationException&,
	          glite::ce::cream_client_api::cream_exceptions::InternalException&,
		  glite::ce::cream_client_api::cream_exceptions::ConnectionTimeoutException&,
		  glite::ce::cream_client_api::cream_exceptions::GridProxyDelegationException&,
		  glite::ce::cream_client_api::cream_exceptions::InvalidArgumentException&,
		  glite::ce::cream_client_api::soap_proxy::auth_ex&);

	  /// Do not use for now
	  virtual void		setConnectionTimeout(const int);
	  /// Do not use for now
	  virtual void          setIOTimeout(const int);
	  /// Do not use for now (it is conceived only for ICE component)
   	  virtual void          setSOAPHeaderID(const std::string&);

	};
      }
    }
  }
}

#endif
